@@ -1,14 +1,23 @@
 from . import constants
+from ._sklearn_adapter import (
+    BaseEstimator,
+    InductiveTransformerMixin,
+    TransformerMixin,
+)
 from ._utils import (
     RandomStateLike,
     _cartesian_product,
     _check_array_key,
     _check_compatible_fdata,
+    _check_compatible_fdatagrid,
     _check_estimator,
     _classifier_fit_depth_methods,
     _classifier_get_classes,
     _classifier_get_depth_methods,
+    _compute_dependence,
+    _DependenceMeasure,
     _evaluate_grid,
+    _fit_feature_transformer,
     _int_to_real,
     _pairwise_symmetric,
     _reshape_eval_points,
@@ -20,3 +29,4 @@ from ._utils import (
     check_is_univariate,
     nquad_vec,
 )
+from ._warping import invert_warping, normalize_scale, normalize_warping
