@@ -1,7 +1,7 @@
+from ._boxplot import BoxplotOutlierDetector
 from ._directional_outlyingness import (
-    DirectionalOutlierDetector,
+    MSPlotOutlierDetector,
     directional_outlyingness_stats,
 )
-from ._iqr import IQROutlierDetector
 from ._outliergram import OutliergramOutlierDetector
 from .neighbors_outlier import LocalOutlierFactor
