@@ -1,5 +1,7 @@
 from . import covariances, kernels, lstsq, metrics, operators, regularization
 from ._math import (
+    cosine_similarity,
+    cosine_similarity_matrix,
     cumsum,
     exp,
     inner_product,
