@@ -1,10 +1,11 @@
 """Metrics, norms and related utilities."""
 
-from ._elastic_metrics import (
-    amplitude_distance,
+from ._angular import angular_distance
+from ._fisher_rao import (
+    _fisher_rao_warping_distance,
+    fisher_rao_amplitude_distance,
     fisher_rao_distance,
-    phase_distance,
-    warping_distance,
+    fisher_rao_phase_distance,
 )
 from ._lp_distances import (
     LpDistance,
@@ -18,5 +19,6 @@ from ._typing import PRECOMPUTED, Metric, Norm
 from ._utils import (
     NormInducedMetric,
     PairwiseMetric,
+    TransformationMetric,
     pairwise_metric_optimization,
 )
