@@ -1,26 +1,28 @@
 from __future__ import annotations
 
 import itertools
+import warnings
 from typing import Any, Generic, Iterable, Optional, Union
 
 import numpy as np
+import scipy.linalg
 from sklearn.base import BaseEstimator
 
-import scipy.linalg
 from skfda.misc.operators import Identity, gramian_matrix
 
 from ...representation import FData
+from ...representation._typing import NDArrayFloat
 from ...representation.basis import Basis
 from ..operators import Operator
 from ..operators._operators import OperatorInput
 
 
-class TikhonovRegularization(
+class L2Regularization(
     BaseEstimator,  # type: ignore
     Generic[OperatorInput],
 ):
     r"""
-    Implements Tikhonov regularization.
+    Implements :math:`L_2` (Tikhonov) regularization.
 
     The penalization term in this type of regularization is the square of the
     :math:`L_2` (Euclidean) norm of a linear operator applied to the function
@@ -38,35 +40,31 @@ class TikhonovRegularization(
     provides several common linear operators.
 
     Parameters:
-        linear_operator: linear operator used for regularization.
-        regularization_parameter: scaling parameter (:math:`\lambda`) of the
-                                  penalization.
+        linear_operator: Linear operator used for regularization. By default
+            the second derivative, which is related with the function
+            curvature, is penalized.
+        regularization_parameter: Scaling parameter (:math:`\lambda`) of the
+            penalization.
 
     Examples:
         Construct a regularization that penalizes the second derivative,
         which is a measure of the curvature of the function.
 
-        >>> from skfda.misc.regularization import TikhonovRegularization
+        >>> from skfda.misc.regularization import L2Regularization
         >>> from skfda.misc.operators import LinearDifferentialOperator
         >>>
-        >>> regularization = TikhonovRegularization(
-        ...                     LinearDifferentialOperator(2))
+        >>> regularization = L2Regularization(
+        ...                     LinearDifferentialOperator(2),
+        ... )
 
-        Construct a regularization that penalizes the identity operator,
-        that is, completely equivalent to the :math:`L_2` regularization (
-        :class:`L2Regularization`).
+        By default the regularization penalizes the identity operator:
 
-        >>> from skfda.misc.regularization import TikhonovRegularization
-        >>> from skfda.misc.operators import Identity
-        >>>
-        >>> regularization = TikhonovRegularization(Identity())
+        >>> regularization = L2Regularization()
 
         Construct a regularization that penalizes the difference between
         the points :math:`f(1)` and :math:`f(0)` of a function :math:`f`.
 
-        >>> from skfda.misc.regularization import TikhonovRegularization
-        >>>
-        >>> regularization = TikhonovRegularization(lambda x: x(1) - x(0))
+        >>> regularization = L2Regularization(lambda x: x(1) - x(0))
 
         Construct a regularization that penalizes the harmonic acceleration
         operator :math:`Lf = \omega^2 D f + D^3 f`, that, when the
@@ -75,20 +73,19 @@ class TikhonovRegularization(
         :math:`\omega` is the angular frequency. This is useful for some
         periodic functions.
 
-        >>> from skfda.misc.regularization import TikhonovRegularization
-        >>> from skfda.misc.operators import LinearDifferentialOperator
         >>> import numpy as np
         >>>
         >>> period = 1
         >>> w = 2 * np.pi / period
-        >>> regularization = TikhonovRegularization(
-        ...                     LinearDifferentialOperator([0, w**2, 0, 1]))
+        >>> regularization = L2Regularization(
+        ...                     LinearDifferentialOperator([0, w**2, 0, 1]),
+        ... )
 
     """
 
     def __init__(
         self,
-        linear_operator: Operator[OperatorInput, Any],
+        linear_operator: Optional[Operator[OperatorInput, Any]] = None,
         *,
         regularization_parameter: float = 1,
     ) -> None:
@@ -98,51 +95,45 @@ class TikhonovRegularization(
     def penalty_matrix(
         self,
         basis: OperatorInput,
-    ) -> np.ndarray:
+    ) -> NDArrayFloat:
         """Return a penalty matrix for ordinary least squares."""
+        linear_operator = (
+            Identity()
+            if self.linear_operator is None
+            else self.linear_operator
+        )
+
         return self.regularization_parameter * gramian_matrix(
-            self.linear_operator,
+            linear_operator,
             basis,
         )
 
 
-class L2Regularization(
-    TikhonovRegularization[Union[np.ndarray, FData, Basis]],
+class TikhonovRegularization(
+    L2Regularization[OperatorInput],
 ):
-    r"""
-    Implements :math:`L_2` regularization.
-
-    The penalization term in this type of regularization is the square of the
-    :math:`L_2` (Euclidean) norm of the function or vector
-
-    .. math::
-            \lambda \| x \|_2^2
-
-    where :math:`\lambda` is a positive real number.
-
-    This is equivalent to Tikhonov regularization (
-    :class:`TikhonovRegularization`) using the identity operator (
-    :class:`Identity`).
-
-    Parameters:
-        regularization_parameter: scaling parameter (:math:`\lambda`) of the
-                                  penalization.
-
-    """
 
     def __init__(
         self,
+        linear_operator: Optional[Operator[OperatorInput, Any]] = None,
         *,
         regularization_parameter: float = 1,
     ) -> None:
+
+        warnings.warn(
+            "Class TikhonovRegularization is deprecated. Use class "
+            "L2Regularization instead.",
+            DeprecationWarning,
+        )
+
         return super().__init__(
-            linear_operator=Identity(),
+            linear_operator=linear_operator,
             regularization_parameter=regularization_parameter,
         )
 
 
 BasisTypes = Union[np.ndarray, FData, Basis]
-Regularization = TikhonovRegularization[Any]
+Regularization = L2Regularization[Any]
 RegularizationLike = Union[
     None,
     Regularization,
@@ -154,7 +145,7 @@ def compute_penalty_matrix(
     basis_iterable: Iterable[BasisTypes],
     regularization_parameter: Union[float, Iterable[float]],
     regularization: RegularizationLike,
-) -> Optional[np.ndarray]:
+) -> Optional[NDArrayFloat]:
     """
     Compute the regularization matrix for a linear differential operator.
 
