@@ -5,118 +5,21 @@ from itertools import combinations
 from typing import Generic, Optional, Sequence, TypeVar, Union
 
 import numpy as np
-from numpy import ndarray
 from scipy.interpolate import lagrange
 from sklearn.base import BaseEstimator, ClassifierMixin, clone
 from sklearn.metrics import accuracy_score
 from sklearn.pipeline import make_pipeline
 from sklearn.utils.validation import check_is_fitted as sklearn_check_is_fitted
 
-from ..._utils import _classifier_fit_depth_methods
+from ..._utils import _classifier_fit_depth_methods, _classifier_get_classes
 from ...exploratory.depth import Depth, ModifiedBandDepth
 from ...preprocessing.dim_reduction.feature_extraction import DDGTransformer
+from ...representation._typing import NDArrayFloat, NDArrayInt
 from ...representation.grid import FData
 
 T = TypeVar("T", bound=FData)
 
 
-class MaximumDepthClassifier(
-    BaseEstimator,  # type: ignore
-    ClassifierMixin,  # type: ignore
-    Generic[T],
-):
-    """Maximum depth classifier for functional data.
-
-    Test samples are classified to the class where they are deeper.
-
-    Parameters:
-        depth_method:
-            The depth class to use when calculating the depth of a test
-            sample in a class. See the documentation of the depths module
-            for a list of available depths. By default it is ModifiedBandDepth.
-    Examples:
-        Firstly, we will import and split the Berkeley Growth Study dataset
-
-        >>> from skfda.datasets import fetch_growth
-        >>> from sklearn.model_selection import train_test_split
-        >>> dataset = fetch_growth()
-        >>> fd = dataset['data']
-        >>> y = dataset['target']
-        >>> X_train, X_test, y_train, y_test = train_test_split(
-        ...     fd, y, test_size=0.25, stratify=y, random_state=0)
-
-        We will fit a Maximum depth classifier
-
-        >>> from skfda.ml.classification import MaximumDepthClassifier
-        >>> clf = MaximumDepthClassifier()
-        >>> clf.fit(X_train, y_train)
-        MaximumDepthClassifier(...)
-
-        We can predict the class of new samples
-
-        >>> clf.predict(X_test) # Predict labels for test samples
-        array([1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1,
-               1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1])
-
-        Finally, we calculate the mean accuracy for the test data
-
-        >>> clf.score(X_test, y_test)
-        0.875
-
-    See also:
-        :class:`~skfda.ml.classification.DDClassifier`
-        :class:`~skfda.ml.classification.DDGClassifier`
-
-    References:
-        Ghosh, A. K. and Chaudhuri, P. (2005b). On maximum depth and
-        related classifiers. Scandinavian Journal of Statistics, 32, 327–350.
-    """
-
-    def __init__(self, depth_method: Optional[Depth[T]] = None) -> None:
-        self.depth_method = depth_method
-
-    def fit(self, X: T, y: ndarray) -> MaximumDepthClassifier[T]:
-        """Fit the model using X as training data and y as target values.
-
-        Args:
-            X: FDataGrid with the training data.
-            y: Target values of shape = (n_samples).
-
-        Returns:
-            self
-        """
-        if self.depth_method is None:
-            self.depth_method = ModifiedBandDepth()
-
-        classes, class_depth_methods = _classifier_fit_depth_methods(
-            X, y, [self.depth_method],
-        )
-
-        self._classes = classes
-        self.class_depth_methods_ = class_depth_methods
-
-        return self
-
-    def predict(self, X: T) -> ndarray:
-        """Predict the class labels for the provided data.
-
-        Args:
-            X: FDataGrid with the test samples.
-
-        Returns:
-             Array of shape (n_samples) with class labels
-                for each data sample.
-        """
-        sklearn_check_is_fitted(self)
-
-        depths = [
-            depth_method.predict(X)
-            for depth_method in self.class_depth_methods_
-        ]
-
-        return self._classes[np.argmax(depths, axis=0)]
-
-
 class DDClassifier(
     BaseEstimator,  # type: ignore
     ClassifierMixin,  # type: ignore
@@ -186,7 +89,7 @@ class DDClassifier(
         self.depth_method = depth_method
         self.degree = degree
 
-    def fit(self, X: T, y: ndarray) -> DDClassifier[T]:
+    def fit(self, X: T, y: NDArrayInt) -> DDClassifier[T]:
         """Fit the model using X as training data and y as target values.
 
         Args:
@@ -210,7 +113,7 @@ class DDClassifier(
             raise ValueError("DDClassifier only accepts two classes.")
 
         dd_coordinates = [
-            depth_method.predict(X)
+            depth_method.transform(X)
             for depth_method in self.class_depth_methods_
         ]
 
@@ -244,7 +147,7 @@ class DDClassifier(
 
         return self
 
-    def predict(self, X: T) -> ndarray:
+    def predict(self, X: T) -> NDArrayInt:
         """Predict the class labels for the provided data.
 
         Args:
@@ -257,7 +160,7 @@ class DDClassifier(
         sklearn_check_is_fitted(self)
 
         dd_coordinates = [
-            depth_method.predict(X)
+            depth_method.transform(X)
             for depth_method in self.class_depth_methods_
         ]
 
@@ -348,7 +251,7 @@ class DDGClassifier(
         (2017) The DDG-classifier in the functional setting. TEST, 26. 119-142.
     """
 
-    def __init__(
+    def __init__(  # noqa: WPS234
         self,
         multivariate_classifier: ClassifierMixin = None,
         depth_method: Union[Depth[T], Sequence[Depth[T]], None] = None,
@@ -356,7 +259,7 @@ class DDGClassifier(
         self.multivariate_classifier = multivariate_classifier
         self.depth_method = depth_method
 
-    def fit(self, X: T, y: ndarray) -> DDGClassifier[T]:
+    def fit(self, X: T, y: NDArrayInt) -> DDGClassifier[T]:
         """Fit the model using X as training data and y as target values.
 
         Args:
@@ -375,7 +278,7 @@ class DDGClassifier(
 
         return self
 
-    def predict(self, X: T) -> ndarray:
+    def predict(self, X: T) -> NDArrayInt:
         """Predict the class labels for the provided data.
 
         Args:
@@ -386,3 +289,110 @@ class DDGClassifier(
                 for each data sample.
         """
         return self._pipeline.predict(X)
+
+
+class _ArgMaxClassifier(
+    BaseEstimator,  # type: ignore
+    ClassifierMixin,  # type: ignore
+):
+    r"""Arg max classifier for multivariate data.
+
+    Test samples are classified to the class that corresponds to the
+    index of the highest coordinate.
+
+    Examples:
+        >>> import numpy as np
+        >>> X = np.array([[1,5], [3,2], [4,1]])
+        >>> y = np.array([1, 0, 0])
+
+        We will fit am ArgMax classifier
+
+        >>> from skfda.ml.classification._depth_classifiers import \
+        ... _ArgMaxClassifier
+        >>> clf = _ArgMaxClassifier()
+        >>> clf.fit(X, y)
+        _ArgMaxClassifier(...)
+
+        We can predict the class of new samples
+
+        >>> clf.predict(X) # Predict labels for test samples
+        array([1, 0, 0])
+    """
+
+    def fit(self, X: NDArrayFloat, y: NDArrayInt) -> _ArgMaxClassifier:
+        """Fit the model using X as training data and y as target values.
+
+        Args:
+            X: Array with the training data.
+            y: Target values of shape = (n_samples).
+
+        Returns:
+            self
+        """
+        classes, _ = _classifier_get_classes(y)
+        self._classes = classes
+        return self
+
+    def predict(self, X: NDArrayFloat) -> NDArrayInt:
+        """Predict the class labels for the provided data.
+
+        Args:
+            X: Array with the test samples.
+
+        Returns:
+            Array of shape (n_samples) with class labels
+                for each data sample.
+        """
+        return self._classes[np.argmax(X, axis=1)]
+
+
+class MaximumDepthClassifier(DDGClassifier[T]):
+    """Maximum depth classifier for functional data.
+
+    Test samples are classified to the class where they are deeper.
+
+    Parameters:
+        depth_method:
+            The depth class to use when calculating the depth of a test
+            sample in a class. See the documentation of the depths module
+            for a list of available depths. By default it is ModifiedBandDepth.
+    Examples:
+        Firstly, we will import and split the Berkeley Growth Study dataset
+
+        >>> from skfda.datasets import fetch_growth
+        >>> from sklearn.model_selection import train_test_split
+        >>> dataset = fetch_growth()
+        >>> fd = dataset['data']
+        >>> y = dataset['target']
+        >>> X_train, X_test, y_train, y_test = train_test_split(
+        ...     fd, y, test_size=0.25, stratify=y, random_state=0)
+
+        We will fit a Maximum depth classifier
+
+        >>> from skfda.ml.classification import MaximumDepthClassifier
+        >>> clf = MaximumDepthClassifier()
+        >>> clf.fit(X_train, y_train)
+        MaximumDepthClassifier(...)
+
+        We can predict the class of new samples
+
+        >>> clf.predict(X_test) # Predict labels for test samples
+        array([1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1,
+               1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1])
+
+        Finally, we calculate the mean accuracy for the test data
+
+        >>> clf.score(X_test, y_test)
+        0.875
+
+    See also:
+        :class:`~skfda.ml.classification.DDClassifier`
+        :class:`~skfda.ml.classification.DDGClassifier`
+
+    References:
+        Ghosh, A. K. and Chaudhuri, P. (2005b). On maximum depth and
+        related classifiers. Scandinavian Journal of Statistics, 32, 327–350.
+    """
+
+    def __init__(self, depth_method: Optional[Depth[T]] = None) -> None:
+        super().__init__(_ArgMaxClassifier(), depth_method)
