@@ -1,5 +1,5 @@
-from . import maxima_hunting
-
+from . import maxima_hunting, mrmr, recursive_maxima_hunting
 from ._rkvs import RKHSVariableSelection
 from .maxima_hunting import MaximaHunting
+from .mrmr import MinimumRedundancyMaximumRelevance
 from .recursive_maxima_hunting import RecursiveMaximaHunting
