@@ -1,19 +1,19 @@
 import abc
 import copy
-import dcor
 import numbers
 import random
 
-import scipy.stats
-import sklearn.base
-import sklearn.utils
-
 import numpy as np
 import numpy.linalg as linalg
 import numpy.ma as ma
+import scipy.stats
+import sklearn.base
+import sklearn.utils
 
+import dcor
+
+from ...._utils import _compute_dependence
 from ....representation import FDataGrid
-from .maxima_hunting import _compute_dependence
 
 
 def _transform_to_2d(t):
@@ -844,17 +844,23 @@ class RecursiveMaximaHunting(
         >>> n_features = 100
         >>>
         >>> def mean_1(t):
-        ...     return (np.abs(t - 0.25)
-        ...             - 2 * np.abs(t - 0.5)
-        ...             + np.abs(t - 0.75))
+        ...     return (
+        ...         np.abs(t - 0.25)
+        ...         - 2 * np.abs(t - 0.5)
+        ...         + np.abs(t - 0.75)
+        ...     )
         >>>
-        >>> X_0 = make_gaussian_process(n_samples=n_samples // 2,
-        ...                             n_features=n_features,
-        ...                             random_state=0)
-        >>> X_1 = make_gaussian_process(n_samples=n_samples // 2,
-        ...                             n_features=n_features,
-        ...                             mean=mean_1,
-        ...                             random_state=1)
+        >>> X_0 = make_gaussian_process(
+        ...     n_samples=n_samples // 2,
+        ...     n_features=n_features,
+        ...     random_state=0,
+        ... )
+        >>> X_1 = make_gaussian_process(
+        ...     n_samples=n_samples // 2,
+        ...     n_features=n_features,
+        ...     mean=mean_1,
+        ...     random_state=1,
+        ... )
         >>> X = skfda.concatenate((X_0, X_1))
         >>>
         >>> y = np.zeros(n_samples)
