@@ -23,7 +23,13 @@ from skfda._utils._utils import _to_array_maybe_ragged
 from ..._utils import _check_array_key, _int_to_real, constants
 from .. import grid
 from .._functional_data import FData
-from .._typing import ArrayLike, DomainRange, GridPointsLike, LabelTupleLike
+from .._typing import (
+    ArrayLike,
+    DomainRange,
+    GridPointsLike,
+    LabelTupleLike,
+    NDArrayInt,
+)
 from ..extrapolation import ExtrapolationLike
 from . import Basis
 
@@ -866,6 +872,27 @@ class FDataBasis(FData):  # noqa: WPS214
     #####################################################################
     # Pandas ExtensionArray methods
     #####################################################################
+    def _take_allow_fill(
+        self: T,
+        indices: NDArrayInt,
+        fill_value: T,
+    ) -> T:
+        result = self.copy()
+        result.coefficients = np.full(
+            (len(indices),) + self.coefficients.shape[1:],
+            np.nan,
+        )
+
+        positive_mask = indices >= 0
+        result.coefficients[positive_mask] = self.coefficients[
+            indices[positive_mask]
+        ]
+
+        if fill_value is not self.dtype.na_value:
+            result.coefficients[~positive_mask] = fill_value.coefficients[0]
+
+        return result
+
     @property
     def dtype(self) -> FDataBasisDType:
         """The dtype for this extension array, FDataGridDType"""
