@@ -25,9 +25,8 @@ from typing import (
 import findiff
 import numpy as np
 import pandas.api.extensions
-from matplotlib.figure import Figure
-
 import scipy.stats.mstats
+from matplotlib.figure import Figure
 
 from .._utils import (
     _check_array_key,
@@ -44,6 +43,7 @@ from ._typing import (
     GridPoints,
     GridPointsLike,
     LabelTupleLike,
+    NDArrayInt,
 )
 from .basis import Basis
 from .evaluator import Evaluator
@@ -1260,7 +1260,7 @@ class FDataGrid(FData):  # noqa: WPS214
 
         new_inputs = [
             i.data_matrix if isinstance(i, FDataGrid)
-            else i for i in inputs
+            else self._get_op_matrix(i) for i in inputs
         ]
 
         outputs = kwargs.pop('out', None)
@@ -1292,6 +1292,28 @@ class FDataGrid(FData):  # noqa: WPS214
     #####################################################################
     # Pandas ExtensionArray methods
     #####################################################################
+
+    def _take_allow_fill(
+        self: T,
+        indices: NDArrayInt,
+        fill_value: T,
+    ) -> T:
+        result = self.copy()
+        result.data_matrix = np.full(
+            (len(indices),) + self.data_matrix.shape[1:],
+            np.nan,
+        )
+
+        positive_mask = indices >= 0
+        result.data_matrix[positive_mask] = self.data_matrix[
+            indices[positive_mask]
+        ]
+
+        if fill_value is not self.dtype.na_value:
+            result.data_matrix[~positive_mask] = fill_value.data_matrix[0]
+
+        return result
+
     @property
     def dtype(self) -> FDataGridDType:
         """The dtype for this extension array, FDataGridDType"""
