@@ -5,31 +5,28 @@ import unittest
 import numpy as np
 
 from skfda import FDataGrid
+from skfda._utils import invert_warping, normalize_warping
 from skfda.datasets import make_multimodal_samples, make_random_warping
+from skfda.exploratory.stats import (
+    _fisher_rao_warping_mean,
+    fisher_rao_karcher_mean,
+)
 from skfda.misc.metrics import (
     PairwiseMetric,
-    amplitude_distance,
+    _fisher_rao_warping_distance,
+    fisher_rao_amplitude_distance,
     fisher_rao_distance,
+    fisher_rao_phase_distance,
     l2_distance,
-    phase_distance,
-    warping_distance,
-)
-from skfda.preprocessing.registration import (
-    ElasticRegistration,
-    invert_warping,
-    normalize_warping,
-)
-from skfda.preprocessing.registration.elastic import (
-    SRSF,
-    elastic_mean,
-    warping_mean,
 )
+from skfda.misc.operators import SRSF
+from skfda.preprocessing.registration import FisherRaoElasticRegistration
 
 metric = PairwiseMetric(l2_distance)
 pairwise_fisher_rao = PairwiseMetric(fisher_rao_distance)
 
 
-class TestElasticRegistration(unittest.TestCase):
+class TestFisherRaoElasticRegistration(unittest.TestCase):
     """Test elastic registration."""
 
     def setUp(self) -> None:
@@ -47,6 +44,33 @@ class TestElasticRegistration(unittest.TestCase):
         t = np.linspace(-3, 3, 9)
         self.dummy_sample = FDataGrid([np.sin(t)], t)
 
+    def test_fit_wrong_dimension(self) -> None:
+        """Checks that template and fit data is compatible."""
+        reg = FisherRaoElasticRegistration(template=self.template)
+
+        unimodal_samples = make_multimodal_samples(
+            n_samples=3,
+            points_per_dim=10,
+            random_state=1,
+        )
+
+        with self.assertRaises(ValueError):
+            reg.fit(unimodal_samples)
+
+    def test_transform_wrong_dimension(self) -> None:
+        """Checks that template and transform data is compatible."""
+        reg = FisherRaoElasticRegistration(template=self.template)
+
+        unimodal_samples = make_multimodal_samples(
+            n_samples=3,
+            points_per_dim=10,
+            random_state=1,
+        )
+
+        reg.fit(self.unimodal_samples)
+        with self.assertRaises(ValueError):
+            reg.transform(unimodal_samples)
+
     def test_to_srsf(self) -> None:
         """Test to srsf."""
         # Checks SRSF conversion
@@ -110,7 +134,7 @@ class TestElasticRegistration(unittest.TestCase):
 
     def test_template_alignment(self) -> None:
         """Test alignment to 1 template."""
-        reg = ElasticRegistration(template=self.template)
+        reg = FisherRaoElasticRegistration(template=self.template)
         register = reg.fit_transform(self.unimodal_samples)
         distances = metric(self.template, register)
 
@@ -118,7 +142,7 @@ class TestElasticRegistration(unittest.TestCase):
 
     def test_one_to_one_alignment(self) -> None:
         """Test alignment to 1 sample to a template."""
-        reg = ElasticRegistration(template=self.template)
+        reg = FisherRaoElasticRegistration(template=self.template)
         register = reg.fit_transform(self.unimodal_samples[0])
         distances = metric(self.template, register)
 
@@ -127,7 +151,7 @@ class TestElasticRegistration(unittest.TestCase):
     def test_set_alignment(self) -> None:
         """Test alignment 3 curves to set with 3 templates."""
         # Should give same result than test_template_alignment
-        reg = ElasticRegistration(template=self.template_rep)
+        reg = FisherRaoElasticRegistration(template=self.template_rep)
         register = reg.fit_transform(self.unimodal_samples)
         distances = metric(self.template, register)
 
@@ -136,7 +160,7 @@ class TestElasticRegistration(unittest.TestCase):
     def test_default_alignment(self) -> None:
         """Test alignment by default."""
         # Should give same result than test_template_alignment
-        reg = ElasticRegistration()
+        reg = FisherRaoElasticRegistration()
         register = reg.fit_transform(self.unimodal_samples)
 
         values = register([-0.25, -0.1, 0, 0.1, 0.25])
@@ -158,7 +182,7 @@ class TestElasticRegistration(unittest.TestCase):
     def test_callable_alignment(self) -> None:
         """Test alignment by default."""
         # Should give same result than test_template_alignment
-        reg = ElasticRegistration(template=elastic_mean)
+        reg = FisherRaoElasticRegistration(template=fisher_rao_karcher_mean)
         register = reg.fit_transform(self.unimodal_samples)
 
         values = register([-0.25, -0.1, 0, 0.1, 0.25])
@@ -178,7 +202,7 @@ class TestElasticRegistration(unittest.TestCase):
 
     def test_simmetry_of_aligment(self) -> None:
         """Check registration using inverse composition."""
-        reg = ElasticRegistration(template=self.template)
+        reg = FisherRaoElasticRegistration(template=self.template)
         reg.fit_transform(self.unimodal_samples)
         warping = reg.warping_
         inverse = invert_warping(warping)
@@ -189,7 +213,7 @@ class TestElasticRegistration(unittest.TestCase):
 
     def test_raises(self) -> None:
         """Test that the assertions raise when appropriate."""
-        reg = ElasticRegistration()
+        reg = FisherRaoElasticRegistration()
 
         # Inverse transform without previous transform
         with np.testing.assert_raises(ValueError):
@@ -201,7 +225,7 @@ class TestElasticRegistration(unittest.TestCase):
             reg.inverse_transform(self.unimodal_samples[0])
 
         # FDataGrid as template with n != 1 and n!= n_samples to transform
-        reg = ElasticRegistration(template=self.unimodal_samples).fit(
+        reg = FisherRaoElasticRegistration(template=self.unimodal_samples).fit(
             self.unimodal_samples[0],
         )
         with np.testing.assert_raises(ValueError):
@@ -209,7 +233,7 @@ class TestElasticRegistration(unittest.TestCase):
 
     def test_score(self) -> None:
         """Test score method of the transformer."""
-        reg = ElasticRegistration()
+        reg = FisherRaoElasticRegistration()
         reg.fit(self.unimodal_samples)
         score = reg.score(self.unimodal_samples)
         np.testing.assert_almost_equal(score, 0.999389)
@@ -217,11 +241,25 @@ class TestElasticRegistration(unittest.TestCase):
     def test_warping_mean(self) -> None:
         """Test the warping_mean function."""
         warping = make_random_warping(start=-1, random_state=0)
-        mean = warping_mean(warping)
+        mean = _fisher_rao_warping_mean(warping)
         values = mean([-1, -0.5, 0, 0.5, 1])
         expected = [[[-1], [-0.376241], [0.136193], [0.599291], [1]]]
         np.testing.assert_array_almost_equal(values, expected)
 
+    def test_linear(self) -> None:
+        grid_points = [i for i in range(10)]
+        data_matrix = np.array([grid_points, grid_points])
+        fd = FDataGrid(
+            data_matrix=data_matrix,
+            grid_points=grid_points,
+        )
+        elastic_registration = FisherRaoElasticRegistration()
+        fd_registered = elastic_registration.fit_transform(fd)
+        np.testing.assert_array_almost_equal(
+            fd_registered.data_matrix[..., 0],
+            data_matrix,
+        )
+
 
 class TestElasticDistances(unittest.TestCase):
     """Test elastic distances."""
@@ -272,34 +310,34 @@ class TestElasticDistances(unittest.TestCase):
             atol=0.01,
         )
 
-    def test_amplitude_distance_limit(self) -> None:
+    def test_fisher_rao_amplitude_distance_limit(self) -> None:
         """Test limit of amplitude distance penalty."""
         f = make_multimodal_samples(n_samples=1, random_state=1)
         g = make_multimodal_samples(n_samples=1, random_state=9999)
 
-        amplitude_limit = amplitude_distance(f, g, lam=1000)
+        amplitude_limit = fisher_rao_amplitude_distance(f, g, lam=1000)
         fr_distance = fisher_rao_distance(f, g)
 
         np.testing.assert_almost_equal(amplitude_limit, fr_distance)
 
-    def test_phase_distance_id(self) -> None:
+    def test_fisher_rao_phase_distance_id(self) -> None:
         """Test of phase distance invariance."""
         f = make_multimodal_samples(n_samples=1, random_state=1)
 
-        phase = phase_distance(f, 2 * f)
+        phase = fisher_rao_phase_distance(f, 2 * f)
 
         np.testing.assert_allclose(phase, 0, atol=1e-7)
 
-    def test_warping_distance(self) -> None:
+    def test_fisher_rao_warping_distance(self) -> None:
         """Test of warping distance."""
         t = np.linspace(0, 1, 1000)
         w1 = FDataGrid([t**5], t)
         w2 = FDataGrid([t**3], t)
 
-        d = warping_distance(w1, w2)
+        d = _fisher_rao_warping_distance(w1, w2)
         np.testing.assert_allclose(d, np.arccos(np.sqrt(15) / 4), atol=1e-3)
 
-        d = warping_distance(w2, w2)
+        d = _fisher_rao_warping_distance(w2, w2)
         np.testing.assert_allclose(d, 0, atol=2e-2)
 
 
