@@ -2,13 +2,15 @@
 import unittest
 
 import numpy as np
+from sklearn.decomposition import PCA
 
+import skfda
 from skfda import FDataBasis, FDataGrid
 from skfda.datasets import fetch_weather
 from skfda.misc.operators import LinearDifferentialOperator
-from skfda.misc.regularization import TikhonovRegularization
+from skfda.misc.regularization import L2Regularization
 from skfda.preprocessing.dim_reduction.feature_extraction import FPCA
-from skfda.representation.basis import Fourier
+from skfda.representation.basis import Basis, BSpline, Fourier
 
 
 class FPCATestCase(unittest.TestCase):
@@ -64,7 +66,7 @@ class FPCATestCase(unittest.TestCase):
 
         fpca = FPCA(
             n_components=n_components,
-            regularization=TikhonovRegularization(
+            regularization=L2Regularization(
                 LinearDifferentialOperator(2),
                 regularization_parameter=1e5,
             ),
@@ -118,7 +120,7 @@ class FPCATestCase(unittest.TestCase):
 
         fpca = FPCA(
             n_components=n_components,
-            regularization=TikhonovRegularization(
+            regularization=L2Regularization(
                 LinearDifferentialOperator(2),
                 regularization_parameter=1e5,
             ),
@@ -169,7 +171,28 @@ class FPCATestCase(unittest.TestCase):
         np.testing.assert_allclose(scores, results, atol=1e-7)
 
     def test_basis_fpca_noregularization_fit_result(self) -> None:
-        """Compare the components in basis against the fda package."""
+        """
+        Compare the components in basis against the fda package.
+
+        Replication code:
+
+            # library(fda)
+            #
+            # data("CanadianWeather")
+            # temp = CanadianWeather$dailyAv[,,1]
+            #
+            # basis = create.fourier.basis(c(0,365), 9)
+            # fdata_temp = Data2fd(1:365, temp, basis)
+            # fpca = pca.fd(fdata_temp, nharm = 3)
+            #
+            # paste(
+            #     round(fpca$harmonics$coefs[,1], 8),
+            #     collapse=", "
+            # ) # first component, others are analogous
+            #
+            # fpca$varprop # explained variance ratio
+
+        """
         n_basis = 9
         n_components = 3
 
@@ -185,19 +208,24 @@ class FPCATestCase(unittest.TestCase):
         # Results obtained using Ramsay's R package
         results = np.array([
             [  # noqa: WPS317
-                0.9231551, 0.1364966, 0.3569451, 0.0092012, -0.0244525,
-                -0.02923873, -0.003566887, -0.009654571, -0.0100063,
+                0.92315509, 0.1395638, 0.35575705,
+                0.00877893, -0.02460726, -0.02932107,
+                -0.0028108, -0.00999328, -0.00966805,
             ],
             [  # noqa: WPS317
-                -0.3315211, -0.0508643, 0.89218521, 0.1669182, 0.24539,
-                0.03548997, 0.037938051, -0.025777507, 0.008416904,
+                -0.33152114, -0.04318338, 0.89258995,
+                0.17111744, 0.24248046, 0.03645764,
+                0.03700911, -0.02547251, 0.00929922,
             ],
             [  # noqa: WPS317
-                -0.1379108, 0.9125089, 0.00142045, 0.2657423, -0.2146497,
-                0.16833314, 0.031509179, -0.006768189, 0.047306718,
+                -0.13791076, 0.91248735, -0.00643356,
+                0.26200806, -0.21919224, 0.16909055,
+                0.02715258, -0.00513581, 0.04751166,
             ],
         ])
 
+        explained_variance_ratio = [0.88958975, 0.08483036, 0.01844100]
+
         # Compare results obtained using this library. There are slight
         # variations due to the fact that we are in two different packages
         # If the sign of the components is not the same the component is
@@ -210,11 +238,62 @@ class FPCATestCase(unittest.TestCase):
         np.testing.assert_allclose(
             fpca.components_.coefficients,
             results,
-            atol=1e-7,
+            atol=0.008,
+        )
+
+        np.testing.assert_allclose(
+            fpca.explained_variance_ratio_,
+            explained_variance_ratio,
+        )
+
+    def test_grid_fpca_fit_sklearn(self) -> None:
+        """Compare the components in grid against the multivariate case."""
+        n_components = 3
+
+        fd_data = fetch_weather()['data'].coordinates[0]
+
+        fpca = FPCA(n_components=n_components, weights=[1] * 365)
+        fpca.fit(fd_data)
+
+        pca = PCA(n_components=n_components)
+        pca.fit(fd_data.data_matrix[..., 0])
+
+        np.testing.assert_allclose(
+            fpca.components_.data_matrix[..., 0],
+            pca.components_,
+        )
+
+        np.testing.assert_allclose(
+            fpca.explained_variance_,
+            pca.explained_variance_,
         )
 
     def test_grid_fpca_fit_result(self) -> None:
-        """Compare the components in grid against the fda.usc package."""
+        """
+        Compare the components in grid against the fda.usc package.
+
+        Replication code:
+
+            # library(fda)
+            # library(fda.usc)
+            #
+            # data("CanadianWeather")
+            # temp = CanadianWeather$dailyAv[,,1]
+            #
+            # fdata_temp = fdata(t(temp))
+            # fpca = fdata2pc(fdata_temp, ncomp = 1)
+            #
+            # paste(
+            #     round(fpca$rotation$data[1,], 8),
+            #     collapse=", "
+            # ) # components
+            # fpca$d[1] # singular value
+            # paste(
+            #     round(fpca$x[,1], 8),
+            #     collapse=", "
+            # ) # transform
+
+        """
         n_components = 1
 
         fd_data = fetch_weather()['data'].coordinates[0]
@@ -227,78 +306,80 @@ class FPCATestCase(unittest.TestCase):
             -0.06958281, -0.07015412, -0.07095115, -0.07185632, -0.07128256,
             -0.07124209, -0.07364828, -0.07297663, -0.07235438, -0.07307498,
             -0.07293423, -0.07449293, -0.07647909, -0.07796823, -0.07582476,
-            -0.07263243, -0.07241871, -0.0718136, -0.07015477, -0.07132331,
-            -0.0711527, -0.07435933, -0.07602666, -0.0769783, -0.07707199,
-            -0.07503802, -0.0770302, -0.07705581, -0.07633515, -0.07624817,
-            -0.07631568, -0.07619913, -0.07568, -0.07595155, -0.07506939,
+            -0.07263243, -0.07241871, -0.07181360, -0.07015477, -0.07132331,
+            -0.07115270, -0.07435933, -0.07602666, -0.07697830, -0.07707199,
+            -0.07503802, -0.07703020, -0.07705581, -0.07633515, -0.07624817,
+            -0.07631568, -0.07619913, -0.07568000, -0.07595155, -0.07506939,
             -0.07181941, -0.06907624, -0.06735476, -0.06853985, -0.06902363,
-            -0.07098882, -0.07479412, -0.07425241, -0.07555835, -0.0765903,
+            -0.07098882, -0.07479412, -0.07425241, -0.07555835, -0.07659030,
             -0.07651853, -0.07682536, -0.07458996, -0.07631711, -0.07726509,
-            -0.07641246, -0.0744066, -0.07501397, -0.07302722, -0.07045571,
+            -0.07641246, -0.07440660, -0.07501397, -0.07302722, -0.07045571,
             -0.06912529, -0.06792186, -0.06830739, -0.06898433, -0.07000192,
-            -0.07014513, -0.06994886, -0.07115909, -0.073999, -0.07292669,
+            -0.07014513, -0.06994886, -0.07115909, -0.07399900, -0.07292669,
             -0.07139879, -0.07226865, -0.07187915, -0.07122995, -0.06975022,
             -0.06800613, -0.06900793, -0.07186378, -0.07114479, -0.07015252,
-            -0.06944782, -0.068291, -0.06905348, -0.06925773, -0.06834624,
+            -0.06944782, -0.06829100, -0.06905348, -0.06925773, -0.06834624,
             -0.06837319, -0.06824067, -0.06644614, -0.06637313, -0.06626312,
-            -0.06470209, -0.0645058, -0.06477729, -0.06411049, -0.06158499,
+            -0.06470209, -0.06450580, -0.06477729, -0.06411049, -0.06158499,
             -0.06305197, -0.06398006, -0.06277579, -0.06282124, -0.06317684,
-            -0.0614125, -0.05961922, -0.05875443, -0.05845781, -0.05828608,
+            -0.06141250, -0.05961922, -0.05875443, -0.05845781, -0.05828608,
             -0.05666474, -0.05495706, -0.05446301, -0.05468254, -0.05478609,
             -0.05440798, -0.05312339, -0.05102368, -0.05160285, -0.05077954,
-            -0.04979648, -0.04890853, -0.04745462, -0.04496763, -0.0448713,
+            -0.04979648, -0.04890853, -0.04745462, -0.04496763, -0.04487130,
             -0.04599596, -0.04688998, -0.04488872, -0.04404507, -0.04420729,
-            -0.04368153, -0.04254381, -0.0411764, -0.04022811, -0.03999746,
-            -0.03963634, -0.03832502, -0.0383956, -0.04015374, -0.0387544,
+            -0.04368153, -0.04254381, -0.04117640, -0.04022811, -0.03999746,
+            -0.03963634, -0.03832502, -0.03839560, -0.04015374, -0.03875440,
             -0.03777315, -0.03830728, -0.03768616, -0.03714081, -0.03781918,
             -0.03739374, -0.03659894, -0.03563342, -0.03658407, -0.03686991,
-            -0.03543746, -0.03518799, -0.03361226, -0.0321534, -0.03050438,
+            -0.03543746, -0.03518799, -0.03361226, -0.03215340, -0.03050438,
             -0.02958411, -0.02855023, -0.02913402, -0.02992464, -0.02899548,
             -0.02891629, -0.02809554, -0.02702642, -0.02672194, -0.02678648,
             -0.02698471, -0.02628085, -0.02674285, -0.02658515, -0.02604447,
-            -0.0245711, -0.02413174, -0.02342496, -0.022898, -0.02216152,
+            -0.02457110, -0.02413174, -0.02342496, -0.02289800, -0.02216152,
             -0.02272283, -0.02199741, -0.02305362, -0.02371371, -0.02320865,
-            -0.02234777, -0.0225018, -0.02104359, -0.02203346, -0.02052545,
+            -0.02234777, -0.02250180, -0.02104359, -0.02203346, -0.02052545,
             -0.01987457, -0.01947911, -0.01986949, -0.02012196, -0.01958515,
-            -0.01906753, -0.01857869, -0.01874101, -0.01827973, -0.017752,
+            -0.01906753, -0.01857869, -0.01874101, -0.01827973, -0.01775200,
             -0.01702056, -0.01759611, -0.01888485, -0.01988159, -0.01951675,
-            -0.01872967, -0.01866667, -0.0183576, -0.01909758, -0.018599,
-            -0.01910036, -0.01930315, -0.01958856, -0.02129936, -0.0216614,
-            -0.0204397, -0.02002368, -0.02058828, -0.02149915, -0.02167326,
+            -0.01872967, -0.01866667, -0.01835760, -0.01909758, -0.01859900,
+            -0.01910036, -0.01930315, -0.01958856, -0.02129936, -0.02166140,
+            -0.02043970, -0.02002368, -0.02058828, -0.02149915, -0.02167326,
             -0.02238569, -0.02211907, -0.02168336, -0.02124387, -0.02131655,
-            -0.02130508, -0.02181227, -0.02230632, -0.02223732, -0.0228216,
+            -0.02130508, -0.02181227, -0.02230632, -0.02223732, -0.02282160,
             -0.02355137, -0.02275145, -0.02286893, -0.02437776, -0.02523897,
-            -0.0248354, -0.02319174, -0.02335831, -0.02405789, -0.02483273,
+            -0.02483540, -0.02319174, -0.02335831, -0.02405789, -0.02483273,
             -0.02428119, -0.02395295, -0.02437185, -0.02476434, -0.02347973,
             -0.02385957, -0.02451257, -0.02414586, -0.02439035, -0.02357782,
             -0.02417295, -0.02504764, -0.02682569, -0.02807111, -0.02886335,
             -0.02943406, -0.02956806, -0.02893096, -0.02903812, -0.02999862,
-            -0.029421, -0.03016203, -0.03118823, -0.03076205, -0.03005985,
+            -0.02942100, -0.03016203, -0.03118823, -0.03076205, -0.03005985,
             -0.03079187, -0.03215188, -0.03271075, -0.03146124, -0.03040965,
             -0.03008436, -0.03085897, -0.03015341, -0.03014661, -0.03110255,
-            -0.03271278, -0.03217399, -0.0331721, -0.03459221, -0.03572073,
-            -0.03560707, -0.03531492, -0.03687657, -0.03800143, -0.0373808,
+            -0.03271278, -0.03217399, -0.03317210, -0.03459221, -0.03572073,
+            -0.03560707, -0.03531492, -0.03687657, -0.03800143, -0.03738080,
             -0.03729927, -0.03748666, -0.03754171, -0.03790408, -0.03963726,
-            -0.03992153, -0.03812243, -0.0373844, -0.0385394, -0.03849716,
-            -0.03826345, -0.03743958, -0.0380861, -0.03857622, -0.04099357,
+            -0.03992153, -0.03812243, -0.03738440, -0.03853940, -0.03849716,
+            -0.03826345, -0.03743958, -0.03808610, -0.03857622, -0.04099357,
             -0.04102509, -0.04170207, -0.04283573, -0.04320618, -0.04269438,
             -0.04467527, -0.04470603, -0.04496092, -0.04796417, -0.04796633,
-            -0.047863, -0.04883668, -0.0505939, -0.05112441, -0.04960962,
-            -0.05000041, -0.04962112, -0.05087008, -0.0521671, -0.05369792,
+            -0.04786300, -0.04883668, -0.05059390, -0.05112441, -0.04960962,
+            -0.05000041, -0.04962112, -0.05087008, -0.05216710, -0.05369792,
             -0.05478139, -0.05559221, -0.05669698, -0.05654505, -0.05731113,
             -0.05783543, -0.05766056, -0.05754354, -0.05724272, -0.05831026,
             -0.05847512, -0.05804533, -0.05875046, -0.06021703, -0.06147975,
-            -0.06213918, -0.0645805, -0.06500849, -0.06361716, -0.06315227,
+            -0.06213918, -0.06458050, -0.06500849, -0.06361716, -0.06315227,
             -0.06306436, -0.06425743, -0.06626847, -0.06615213, -0.06881004,
-            -0.06942296, -0.06889225, -0.06868663, -0.0678667, -0.06720133,
+            -0.06942296, -0.06889225, -0.06868663, -0.06786670, -0.06720133,
             -0.06771172, -0.06885042, -0.06896979, -0.06961627, -0.07211988,
             -0.07252956, -0.07265559, -0.07264195, -0.07306334, -0.07282035,
             -0.07196505, -0.07210595, -0.07203942, -0.07105821, -0.06920599,
             -0.06892264, -0.06699939, -0.06537829, -0.06543323, -0.06913186,
             -0.07210039, -0.07219987, -0.07124228, -0.07065497, -0.06996833,
-            -0.0674457, -0.06800847, -0.06784175, -0.06592871, -0.06723401,
+            -0.06744570, -0.06800847, -0.06784175, -0.06592871, -0.06723401,
         ])
 
+        singular_value = 728.9945
+
         # Compare results obtained using this library. There are slight
         # variations due to the fact that we are in two different packages
         # If the sign of the components is not the same the component is
@@ -314,8 +395,18 @@ class FPCATestCase(unittest.TestCase):
             rtol=1e-6,
         )
 
+        np.testing.assert_allclose(
+            fpca.singular_values_,
+            singular_value,
+        )
+
     def test_grid_fpca_transform_result(self) -> None:
-        """Compare the scores in grid against the fda.usc package."""
+        """
+        Compare the scores in grid against the fda.usc package.
+
+        See test_grid_fpca_fit_result for the replication code.
+
+        """
         n_components = 1
 
         fd_data = fetch_weather()['data'].coordinates[0]
@@ -326,21 +417,18 @@ class FPCATestCase(unittest.TestCase):
 
         # results obtained
         results = np.array([  # noqa: WPS317
-            [-77.05020176], [-90.56072204], [-82.39565947],
-            [-114.45375934], [-69.99735931], [-64.44894047],
-            [135.58336775], [-14.93460852], [0.75024737],
-            [-36.4781038], [-42.35637749], [-73.98910492],
-            [-67.11253749], [-103.68269798], [-104.65948079],
-            [-7.42817782], [7.48125036], [56.29792942],
-            [181.00258791], [-3.53294736], [37.94673912],
-            [124.43819913], [-7.04274676], [-49.61134859],
-            [-136.86256785], [-184.03502398], [-181.72835749],
-            [-51.06323208], [-137.85606731], [50.10941466],
-            [151.68118097], [159.01360046], [217.17981302],
-            [234.40195237], [345.39374006],
+            -76.43217603, -90.02095494, -81.80476223, -113.69868192,
+            -69.54664059, -64.15532621, 134.93536815, -15.00125409,
+            0.60569550, -36.37615052, -42.18300642, -73.71660038,
+            -66.88119544, -103.15419038, -104.12065321, -7.49806764,
+            7.14456774, 55.76321474, 180.16351452, -3.76283358,
+            37.49075282, 123.73187622, -7.05384351, -49.36562021,
+            -136.37428322, -183.00666524, -180.64875116, -50.94411798,
+            -136.95768454, 49.83695668, 150.67710532, 158.20189044,
+            216.43002289, 233.53770292, 344.18479151,
         ])
 
-        np.testing.assert_allclose(scores, results, rtol=1e-6)
+        np.testing.assert_allclose(scores.ravel(), results, rtol=0.25)
 
     def test_grid_fpca_regularization_fit_result(self) -> None:
         """Compare the components in grid against the fda.usc package."""
@@ -351,7 +439,7 @@ class FPCATestCase(unittest.TestCase):
         fpca = FPCA(
             n_components=n_components,
             weights=[1] * 365,
-            regularization=TikhonovRegularization(
+            regularization=L2Regularization(
                 LinearDifferentialOperator(2),
             ),
         )
@@ -449,6 +537,134 @@ class FPCATestCase(unittest.TestCase):
             rtol=1e-2,
         )
 
+    def draw_one_random_fun(
+        self,
+        basis: Basis,
+        random_state: np.random.RandomState,
+    ) -> FDataBasis:
+        """Draw a true function in a given basis with random coef."""
+        coef = random_state.uniform(-10, 10, size=basis.n_basis)
+        return FDataBasis(
+            basis=basis,
+            coefficients=coef,
+        )
+
+    def test_grid_fpca_inverse_transform(self) -> None:
+        """Compare the reconstructions.data_matrix to fitting data."""
+        random_state = np.random.RandomState(seed=42)
+
+        def test_vs_dim(
+            n_samples: int,
+            n_grid: int,
+            base_fun: FDataBasis,
+        ) -> None:
+            """Test function w.r.t n_samples, n_grid."""
+            # Random offsetting base_fun and form dataset fd_random
+            offset = random_state.uniform(-5, 5, size=n_samples)
+
+            fd_random = FDataBasis(
+                basis=base_fun.basis,
+                coefficients=base_fun.coefficients * offset[:, np.newaxis],
+            ).to_grid(np.linspace(0, 1, n_grid))
+
+            # Take the allowed maximum number of components
+            # In almost high dimension: n_components=n_samples-1 < n_samples
+            # In low dimension: n_components=n_grid << n_samples
+            fpca = FPCA(
+                n_components=min(n_samples - 1, n_grid),
+            )
+
+            # Project the non-random dataset on FPCs
+            pc_scores_fd_random_all_equal = fpca.fit_transform(
+                fd_random,
+            )
+
+            # Project the pc scores back to the input functional space
+            fd_random_hat = fpca.inverse_transform(
+                pc_scores_fd_random_all_equal,
+            )
+
+            # Compare fitting data to the reconstructed ones
+            np.testing.assert_allclose(
+                fd_random.data_matrix,
+                fd_random_hat.data_matrix,
+            )
+
+        # Low dimensional case (n_samples>n_grid)
+        n_samples = 1000
+        n_grid = 100
+        bsp = BSpline(
+            domain_range=(0, 50),
+            n_basis=100,
+            order=3,
+        )
+        true_fun = self.draw_one_random_fun(bsp, random_state)
+        test_vs_dim(n_samples=n_samples, n_grid=n_grid, base_fun=true_fun)
+
+        # (almost) High dimensional case (n_samples<n_grid)
+        n_samples = 100
+        n_grid = 1000
+        bsp = BSpline(
+            domain_range=(0, 50),
+            n_basis=100,
+            order=3,
+        )
+        true_fun = self.draw_one_random_fun(bsp, random_state)
+        test_vs_dim(n_samples=n_samples, n_grid=n_grid, base_fun=true_fun)
+
+    def test_basis_fpca_inverse_transform(self) -> None:
+        """Compare the coef reconstructions to fitting data."""
+        random_state = np.random.RandomState(seed=42)
+
+        def test_vs_dim(n_samples: int, base_fun: FDataBasis) -> None:
+            """Test function w.t.t n_samples and basis."""
+            # Random offsetting base_fun and form dataset fd_random
+            offset = random_state.uniform(-5, 5, size=n_samples)
+
+            fd_random = FDataBasis(
+                basis=base_fun.basis,
+                coefficients=base_fun.coefficients * offset[:, np.newaxis],
+            )
+
+            # Take the allowed maximum number of components
+            # In almost high dimension: n_components=n_samples-1 < n_samples
+            # In low dimension: n_components=n_basis<<n_samples
+            fpca = FPCA(n_components=min(n_samples - 1, base_fun.n_basis))
+
+            # Project non-random dataset on fitted FPCs
+            pc_scores = fpca.fit_transform(fd_random)
+
+            # Project back pc_scores to functional input space
+            fd_random_hat = fpca.inverse_transform(pc_scores)
+
+            # Compare fitting data to the reconstructed ones
+            np.testing.assert_allclose(
+                fd_random.coefficients,
+                fd_random_hat.coefficients,
+            )
+
+        # Low dimensional case: n_basis<<n_samples
+        n_samples = 1000
+        n_basis = 100
+        bsp = BSpline(
+            domain_range=(0, 50),
+            n_basis=n_basis,
+            order=3,
+        )
+        true_fun = self.draw_one_random_fun(bsp, random_state)
+        test_vs_dim(n_samples=n_samples, base_fun=true_fun)
+
+        # Case n_samples<n_basis
+        n_samples = 10
+        n_basis = 100
+        bsp = BSpline(
+            domain_range=(0, 50),
+            n_basis=n_basis,
+            order=3,
+        )
+        true_fun = self.draw_one_random_fun(bsp, random_state)
+        test_vs_dim(n_samples=n_samples, base_fun=true_fun)
+
 
 if __name__ == '__main__':
     unittest.main()
