@@ -11,12 +11,16 @@ from sklearn.linear_model import Ridge
 from sklearn.model_selection._split import train_test_split
 
 import skfda
-from skfda.misc.operators import LinearDifferentialOperator, gramian_matrix
+from skfda.misc.operators import (
+    Identity,
+    LinearDifferentialOperator,
+    gramian_matrix,
+)
 from skfda.misc.operators._linear_differential_operator import (
     _monomial_evaluate_constant_linear_diff_op,
 )
 from skfda.misc.operators._operators import gramian_matrix_numerical
-from skfda.misc.regularization import L2Regularization, TikhonovRegularization
+from skfda.misc.regularization import L2Regularization
 from skfda.ml.regression import LinearRegression
 from skfda.representation.basis import (
     Basis,
@@ -224,6 +228,42 @@ class TestLinearDifferentialOperatorRegularization(unittest.TestCase):
         )
 
 
+class TestDefaultL2Regularization(unittest.TestCase):
+    """Test default value of L2 regularization."""
+
+    def test_basis_default(self) -> None:
+        """Test that in basis smoothing."""
+        data_matrix = np.linspace([0, 1, 2, 3], [1, 2, 3, 4], 100)
+
+        fd = skfda.FDataGrid(data_matrix.T)
+
+        smoother = skfda.preprocessing.smoothing.BasisSmoother(
+            basis=skfda.representation.basis.BSpline(
+                n_basis=10,
+                domain_range=fd.domain_range,
+            ),
+            regularization=L2Regularization(),
+        )
+
+        smoother2 = skfda.preprocessing.smoothing.BasisSmoother(
+            basis=skfda.representation.basis.BSpline(
+                n_basis=10,
+                domain_range=fd.domain_range,
+            ),
+            regularization=L2Regularization(
+                Identity(),
+            ),
+        )
+
+        fd_basis = smoother.fit_transform(fd)
+        fd_basis2 = smoother2.fit_transform(fd)
+
+        np.testing.assert_allclose(
+            fd_basis.data_matrix,
+            fd_basis2.data_matrix,
+        )
+
+
 class TestEndpointsDifferenceRegularization(unittest.TestCase):
     """Test regularization with a callable."""
 
@@ -238,7 +278,7 @@ class TestEndpointsDifferenceRegularization(unittest.TestCase):
                 n_basis=10,
                 domain_range=fd.domain_range,
             ),
-            regularization=TikhonovRegularization(
+            regularization=L2Regularization(
                 lambda x: x(1)[:, 0] - x(0)[:, 0],
             ),
             smoothing_parameter=10000,
